package jalview;

import java.util.*;
import java.awt.*;
import scooby.annotation.*;
import scooby.seq.io.*;

import javax.swing.*;

public class ClusterHits {

    FeaturePair[] features;
    float      distance[][];
    int        noClus;
    Vector     cluster;
    int        done[];
    int        nofeatures;
    int        mini;
    int        minj;
    float      ri;
    float      rj;
    Vector     node;
    TreeFile   tf;

    public ClusterHits(FeaturePair[] features) {

	this.features = features;
	this.node     = new Vector();

	int i = 0;
	done  = new int[features.length];
	
	while (i < features.length && features[i] != null) {
	    done[i] = 0;
	    i++;
	}

	nofeatures   = i++; 

        System.out.println("Finding distances");

	distance = findDistances();

	System.out.println("Making leaves");
	makeLeaves();

	
	noClus   = cluster.size();

	System.out.println("Clustering");
	cluster();
	
    }
    
    public float[][] findDistances() {

	float[][] distance = new float[nofeatures][nofeatures];
	
	for (int i = 0; i < nofeatures-1; i++) {
	    for (int j = i; j < nofeatures; j++) {
		
		if (j==i) {
		    distance[i][i] = 0;
		} else {
		    FeaturePair fp1 = features[i];
		    FeaturePair fp2 = features[j];

		    float pos1a = (float)((fp1.getEnd() + fp1.getStart())/2);
		    float pos2a = (float)((fp2.getEnd() + fp2.getStart())/2);

		    float pos1b = (float)((fp1.getHend() + fp1.getHstart())/2);
		    float pos2b = (float)((fp2.getHend() + fp2.getHstart())/2);

		    distance[i][j] = (float)Math.sqrt((pos2a - pos1a)*(pos2a - pos1a) + (pos2b - pos1b)*(pos2b - pos1b));
		    distance[j][i] = distance[i][j];
		}
	    }
	}

	return distance;

    }


    public void makeLeaves() {

    cluster = new Vector();

    for (int i=0; i < nofeatures; i++) {

	SequenceNode sn = new SequenceNode();
	sn.element = features[i];

	node.addElement(sn);

	int[] value = new int[1];

	value[0] = i;

	Cluster c = new Cluster(value);

	cluster.addElement(c);

    }
    
}


public void cluster() {

    while (noClus > 2) {
        System.out.println("Number of clusters = " + noClus);
	float   mind = findMinDistance();
	Cluster c    = joinClusters(mini,minj);

	done[minj] = 1;

	cluster.setElementAt(null,minj);
	cluster.setElementAt(c,mini);
	
	noClus--;

    }

    boolean onefound = false;

    int one = -1;
    int two = -1;

    for (int i=0; i < nofeatures; i++) {
        
	if (done[i] != 1) {

	    if (onefound == false) {
		two = i;
		onefound = true;
	    } else {
		one = i;
	    }
	}
    }
    
    Cluster c = joinClusters(one,two);

    tf = new TreeFile((SequenceNode)(node.elementAt(one)));

  }



  public Cluster joinClusters(int i, int j) {

    float dist = distance[i][j];

    System.out.println("Dist = " + dist + " " + i + " " + j + " " + cluster.size());
    int noi = ((Cluster)cluster.elementAt(i)).value.length;
    int noj = ((Cluster)cluster.elementAt(j)).value.length;

    int[] value = new int[noi + noj];

    for (int ii = 0; ii < noi;ii++) {
	value[ii] =  ((Cluster)cluster.elementAt(i)).value[ii];
    }

    for (int ii = noi; ii < noi+ noj;ii++) {
	value[ii] =  ((Cluster)cluster.elementAt(j)).value[ii-noi];
    }
    
    Cluster c = new Cluster(value);

    ri = findr(i,j);
    rj = findr(j,i);

    findClusterDistance(i,j);


    // Make a new node

    SequenceNode sn = new SequenceNode();

    sn.left = (SequenceNode)(node.elementAt(i));
    sn.right =(SequenceNode)(node.elementAt(j));

    SequenceNode tmpi = (SequenceNode)(node.elementAt(i));
    SequenceNode tmpj = (SequenceNode)(node.elementAt(j));

    findNewDistances(tmpi,tmpj,dist);

    tmpi.parent = sn;
    tmpj.parent = sn;

    node.setElementAt(sn,i);

    return c;

  }

  public void findNewDistances(SequenceNode tmpi,SequenceNode tmpj,float dist) {

    float ih = 0;
    float jh = 0;

    SequenceNode sni = tmpi;
    SequenceNode snj = tmpj;

    while (sni != null) {

      ih = ih + sni.dist;
      sni = (SequenceNode)sni.left;

     }

    while (snj != null) {

      jh = jh + snj.dist;
      snj = (SequenceNode)snj.left;

     }

    tmpi.dist = (dist/2 - ih);
    tmpj.dist = (dist/2 - jh);
    
  }

  public void findClusterDistance(int i, int j) {

    int noi = ((Cluster)cluster.elementAt(i)).value.length;
    int noj = ((Cluster)cluster.elementAt(j)).value.length;

    // New distances from cluster to others

    float[] newdist = new float[nofeatures];

    for (int l = 0; l < nofeatures; l++) {

	if ( l != i && l != j) {
	    newdist[l] = (distance[i][l] * noi + distance[j][l] * noj)/(noi + noj);
	} else {
	    newdist[l] = 0;
	}
    }

    for (int ii=0; ii < nofeatures;ii++) {
	distance[i][ii] = newdist[ii];
	distance[ii][i] = newdist[ii];
    }
    
  }


public float findr(int i, int j) {

    float tmp = 1;

    for (int k=0; k < nofeatures;k++) {

      if (k!= i && k!= j && done[k] != 1) {

	tmp = tmp + distance[i][k];

      }

    }

    if (noClus > 2) {

      tmp = tmp/(noClus - 2);

    } 
    return tmp;

  }

  public float findMinDistance() {

      float min = 100000000000;
      
      for (int i=0; i < nofeatures-1;i++) {

	  for (int j = i+1; j < nofeatures;j++) {
	      if (done[i] != 1 && done[j] != 1) {

		  if (distance[i][j] < min) {
		      mini = i;
		      minj = j;

		      min = distance[i][j];
		      
		  }
		  
	      }
	      
	  }
	  
      }
      return min;
      
  }

  public static void main(String[] args) {

    try {

	scooby.seq.io.GFFFile gff = new scooby.seq.io.GFFFile(args[0],"File");

	FeaturePair[] fp = new FeaturePair[gff.seqs.size()];

	for (int i=0; i < gff.seqs.size();i++) {

	    fp[i] = (FeaturePair)gff.seqs.elementAt(i);
	}

	ClusterHits clus = new ClusterHits(fp);


  	clus.tf.reCount(clus.tf.top);
  	clus.tf.findHeight(clus.tf.top);
	//  	clus.tf.printNode(clus.tf.top);

	System.out.println("Grouping");
	clus.tf.groupNodes(clus.tf.top,Float.valueOf(args[1]).floatValue());

	System.out.println("Groups are " + clus.tf.groups.size() + " " + clus.tf.maxheight);

	for (int i=0; i < clus.tf.groups.size(); i++) {
	    System.out.println();

	    System.out.println("Group " + i);

	    SequenceNode node = (SequenceNode)clus.tf.groups.elementAt(i);
	    
	    int count = node.countLeaves();

	    System.out.println("Number of leaves = " + count);
	    
	    int mincount = 3;
	    if (args.length > 2) {
		mincount = Integer.parseInt(args[2]);
	    }
	    if (count >= mincount) {
		clus.tf.printNode(node);
	    }
	}
	if (args.length > 3 && args[3].equals("-p")) {
	    Frame f = new Frame();
	    
	    f.setLayout(new BorderLayout());
	    
	    JScrollPane p = new JScrollPane();
	    p.setPreferredSize(new Dimension(600,600));
	    
	    MyCanvas mc = new MyCanvas(clus.tf);
	    
	    p.setDoubleBuffered(true);
	    mc.setDoubleBuffered(true);
	    
	    mc.setPreferredSize(new Dimension(700,1000));
	    mc.setOpaque(true);
	    p.getViewport().add(mc);
	    
	    f.resize(600,600);
	    f.add("Center",p);
	    f.show();
	    
	    mc.validate();

	    clus.tf.draw(p.getGraphics(),mc.getPreferredSize().width,mc.getPreferredSize().height);
	}
    } catch (java.io.IOException e) {

      System.out.println("Exception : " + e);

    }

  }

}

class Cluster {

    int[] value;


  public Cluster(int[] value) {

    this.value = value;

  }

}

class MyCanvas extends JPanel {

    TreeFile tf;
    Image   img;
    Graphics gg;



  public MyCanvas(TreeFile tf) {

    super();

    this.tf = tf;

  }

  public void paint(Graphics g) {

    if (size() != null) {
	img = createImage(size().width,size().height);
	tf.draw(img.getGraphics(),size().width,size().height);

	g.drawImage(img,0,0,this);    
    } else {
	img = createImage(500,500);
	
	tf.draw(g,500,500);

	g.drawImage(img,0,0,this);    
    }

  }

    public void update(Graphics g) {
	paint(g);
    }

    public void repaint() {
	if (this.getGraphics() != null) {
	    update(this.getGraphics());
	}
    }

  

}

  

