

package jalview;

import java.util.*;
import java.io.*;
import java.awt.*;
import javax.swing.*;

public class EstAlign extends AlignSeq {

    int  lookup[][] = ResidueProperties.DNA;
    int[] donor;
    int[] acceptor;
    int[][] point;
    int intronExon = 5000;
    int splice     = -50;
    int intron     = 0;
    int[] mostprobi;
    int[] mostprobj;

  public EstAlign(Sequence s1, Sequence s2,String type){
    super(s1,s2,type);
    donor    = findDonors();
    acceptor = findAcceptors();
  }


    
  public int[] findDonors() {
    int[] donor = new int[s2str.length()];

    for (int i = 0; i < (s2str.length()-2); i++) {
      if (s2str.substring(i,i+2).equals("GT")) {
	  donor[i] = 0;
      } else {
	  donor[i] = 1;
      }
    }

    return donor;

  }

  

  public int[] findAcceptors() {
    int[] acceptor = new int[s2str.length()];

    acceptor[0] = 1;
    acceptor[1] = 1;

    for (int i = 2; i < s2str.length(); i++) {

      if (s2str.substring(i-2,i).equals("AG")) {
	acceptor[i] = 0;

      } else {

	acceptor[i] = 1;

      }

    }

    return acceptor;

  }

  public void calcScoreMatrix() {

    int n = seq1.length;
    int m = seq2.length;


    point = new int[n][m];

    // top left hand element
    E = new int[n][m];

    score[0][0] =  lookup[seq1[0]][seq2[0]] * 10;
    E[0][0] = 0;
    // Calculate the top row first

    for (int j = 1; j < m; j++) {
      // What should these values be? 0 maybe	

	score[0][j] = lookup[seq1[0]][seq2[j]] * 10;
	E[0][j] = 0;

    }
    

    // Now do the left hand column

    for (int i = 1; i < n; i++) { 
	score[i][0] = lookup[seq1[i]][seq2[0]] * 10;
	E[i][0] = 0 ;
    }

    // Now do all the other rows

    for (int i = 1; i < n; i++) {
      for (int j = 1; j < m; j++) {


	  E[i][j] = max(score[i][j-1] - donor[j]*intronExon + splice,E[i][j-1] + intron);

	  int e = score[i][j-1] - gapOpen;
	  int f = score[i-1][j] - gapOpen;
	  int h = E[i][j-1] - acceptor[j]*intronExon  + splice +  lookup[seq1[i]][seq2[j]]*10;

	  score[i][j] = max(score[i-1][j-1] + lookup[seq1[i]][seq2[j]]*10,
			    e,f,E[i][j],h,i,j);

	  if (donor[j] == 0) {
	      //	      System.out.println("Score " + i + " " + j + " " + score[i][j] + " " + E[i][j] + " " + E[i][j-1]);
	  }
      }
    }
    if (n < 20 && m < 20) {
	printScoreMatrix(score,point);
    }
  }

    public void showMatrix(int mat[][],String title,int type,int[] probi, int[] probj) {

	Frame f = new Frame(title);

	JScrollPane sp = new JScrollPane();
	MatrixPanel mp = new MatrixPanel(mat,type,probi,probj);

	sp.add(mp);

	f.setSize(500,500);
	f.add(mp);
	f.show();
    }
    public void traceAlignment() {
	mostprobi = new int[seq1.length*seq2.length];
	mostprobj = new int[seq2.length*seq2.length];
    // Find the maximum score along the rhs or bottom row

    int max = -9999;

    for (int i = 0; i < seq1.length; i++) {
      if (score[i][seq2.length - 1] > max ) {
	max = score[i][seq2.length - 1];
	maxi = i;
	maxj = seq2.length-1;

      }
    }

    for (int j = 0; j < seq2.length; j++) {
      if (score[seq1.length - 1][j] > max ) {
	max = score[seq1.length - 1][j];
	maxi = seq1.length-1;
	maxj = j;
      }
    }


    System.out.println(maxi + " " + maxj + " " + score[maxi][maxj] + " " + intToStr[seq1[maxi]] + " " + intToStr[seq2[maxj]]);

    int i = maxi;
    int j = maxj;
    int trace;

    maxscore = score[i][j] / 10;

    seq1end = maxi+1;
    seq2end = maxj+1;

    aseq1 = new int[seq1.length + seq2.length];
    aseq2 = new int[seq1.length + seq2.length];
   
    count = seq1.length + seq2.length - 1;

    while (i>0 && j >0) {

	if (aseq1[count] != defInt && i >=0) {
	  
	    aseq1[count] = seq1[i];
	    astr1 = intToStr[seq1[i]] + astr1;

	}

	if (aseq2[count] != defInt && j > 0) {
	    
	    aseq2[count] = seq2[j];
	    astr2 = intToStr[seq2[j]] + astr2;
	    
	}
	
	//trace = findTrace(i,j);

	mostprobi[count] = i;
	mostprobj[count] = j;
	//	System.out.println("Most prob " + count + " " + i + " " + j);
	trace = point[i][j];

	if (trace == 10) { 
	    i--;
	    j--;
	    
	} else  if (trace == 20) { 
	    i--;

	    aseq1[count] = defInt;
	    astr2 = "*" + astr2.substring(1);

	} else  if (trace == 30 || trace == 40 || trace == 50) { 

	    j--;

	    aseq2[count] = defInt;

	    if (trace == 30) {
		astr1 = "." + astr1.substring(1);
	    } else if (trace == 40) {
		astr1 = "^" + astr1.substring(1);
	    } 
	}

	count--;

      }

    seq1start = i+1;
    seq2start = j+1;

    if (aseq1[count] != defInt) {
     aseq1[count] = seq1[i];
     astr1 = intToStr[seq1[i]] + astr1;
    }

    if (aseq2[count] != defInt) {
      aseq2[count] = seq2[j];
      astr2 = intToStr[seq2[j]] + astr2;
    }

    System.out.println(astr1);
    System.out.println(astr2);


    showMatrix(score,"Scores",MatrixPanel.GRADIENT,mostprobi,mostprobj);
    //    showMatrix(E,"E",MatrixPanel.GRADIENT,mostprobi,mostprobj);
    //showMatrix(point,"Traceback",MatrixPanel.BLOCK,mostprobi,mostprobj);

    }

  public void printAlignment() {
    // Find the biggest id length for formatting purposes
    int maxid = s1.getName().length();

    if (s2.getName().length() > maxid) {
      maxid = s2.getName().length();
    }

    int len      = 72 - maxid - 1;    
    int nochunks = ((aseq1.length - count) / len) + 1;
    int overlap  = 0;

    pid = 0;

    output = output + ("Score = " + score[maxi][maxj] + "\n");
    output = output + ("Length of alignment = " + (aseq1.length-count) + "\n");
    output = output + ("Sequence ");
    output = output + (new Format("%" + maxid + "s").form(s1.getName()));
    output = output + (" :  " + seq1start + " - " + seq1end + " (Sequence length = " + s1str.length() + ")\n");
    output = output + ("Sequence ");
    output = output + (new Format("%" + maxid + "s").form(s2.getName()));
    output = output + (" :  " + seq2start + " - " + seq2end + " (Sequence length = " + s2str.length() + ")\n\n");

      for (int j = 0; j < nochunks; j++) {
	// Print the first aligned sequence
	  output = output + (new Format("%" + (maxid) + "s").form(s1.getName()) + " ");
	  for (int i = 0; i < len ; i++) {
	      if ((count + i + j*len) < aseq1.length) {
		  output = output + (new Format("%s").form(intToStr[aseq1[count + i + j*len]]));
	      }
	  }

	  output = output + ("\n");
	  output = output + (new Format("%" + (maxid) + "s").form(" ") + " ");

	// Print out the matching chars

	for (int i = 0; i < len ; i++) {
	    if ((count + i + j*len) < aseq1.length) {
	    if ( intToStr[aseq1[count+i+j*len]].equals(intToStr[aseq2[count+i+j*len]]) && !intToStr[aseq1[count+i+j*len]].equals("-")) {
	      pid++;
	      output = output + ("|");
	    } else if (type.equals("pep")) {
	      if (ResidueProperties.getPAM250(intToStr[aseq1[count+i+j*len]],intToStr[aseq2[count+i+j*len]]) > 0) {
		output  = output + (".");
	      } else {
		output = output + (" ");
	      }
	    } else {
	      output = output + (" ");
	    }
	  }
	}

	// Now print the second aligned sequence

	output = output + ("\n");
	output = output + (new Format("%" + (maxid) + "s").form(s2.getName()) + " " );

	for (int i = 0; i < len ; i++) {
	    if ((count + i + j*len) < aseq1.length) {
	    //    output = output + (new Format("%s").form(intToStr[aseq2[count + i + j*len]]));
	  }
	}

	if ((j+1)*len < astr2.length()) {
	  output = output + astr2.substring(j*len,(j+1)*len);
	} else {
	  output = output + astr2.substring(j*len);
	}
	output = output + ("\n\n");
      }

      pid = pid/(float)(aseq1.length-count)*100;

      output = output + (new Format("Percentage ID = %2.2f\n\n").form(pid));

      

    }

   



  public int max(int i1, int i2, int i3, int i4,int i5,int i,int j) {
      // match, jinsert, iinsert, donor, acceptor
      int max = i1;
      int p = 10;
      if (i2 > i1) {max = i2; p =20;}
      if (i3 > max) {max = i3; p = 30;}
      if (i4 >= max) {max = i4; p = 40;}
      if (i5 >= max) {max = i5; p = 50;}

      point[i][j] = p;
      return max;
  }

  public static void main(String[] args) {

      Sequence[] s = new Sequence[2];

      try {
	  if (args.length != 2) {
	      System.out.println("args: <dnafile> <estfile> ");
	      System.exit(0);
	  }

	  FastaFile fa1 = new FastaFile(args[0],"File");
	  FastaFile fa2 = new FastaFile(args[1],"File");

	  s[0] = (Sequence)fa1.seqs.elementAt(0);
	  s[1] = (Sequence)fa2.seqs.elementAt(0);


      } catch (Exception e) {
	  System.out.println("Exception : " + e);
      }

      if (s[0] != null && s[1] != null) {
	  EstAlign as = new EstAlign(s[0],s[1],"dna");

	  as.gapExtend = 40;
	  as.gapOpen  = 100;
	  as.calcScoreMatrix();
	  as.traceAlignment();
	  as.printAlignment();
	  
	  s[0] = new Sequence(as.s1.getName(),as.astr1,0,0);
	  s[1] = new Sequence(as.s2.getName(),as.astr2,0,0);

	  System.out.println(as.output + "\nScore = " + as.maxscore);
      }
  }
}

















