/* Jalview - a java multiple alignment editor
 * Copyright (C) 1998  Michele Clamp
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package jalview;

import java.awt.*;
import java.applet.Applet;
import java.util.*;
import java.net.*;
import java.io.*;
import MCview.*;

public class Jtml {
    public DrawableAlignment da;
    public Hashtable styles;

    public Jtml(DrawableAlignment da) {
	this.da = da;

	styles = new Hashtable();

	Hashtable colhash = ClustalxColourScheme.colhash;

	styles.put(colhash.get("RED"),   "a");
	styles.put(colhash.get("BLUE"),  "b");
	styles.put(colhash.get("GREEN"), "c");
	styles.put(colhash.get("ORANGE"),"d");
	styles.put(colhash.get("CYAN"),  "e");
	styles.put(colhash.get("PINK"),  "f");
	styles.put(colhash.get("MAGENTA"),  "f");
	styles.put(colhash.get("YELLOW"), "g");

	Hashtable sshash = SecondaryColourScheme.colhash;

	styles.put(sshash.get("MAGENTA"),"h");
	styles.put(sshash.get("YELLOW"),"i");
    }
    
    public void getHtml(int linelength,String stylesheet) {
	System.out.println("<HTML><HEAD><LINK REL=\"stylesheet\" HREF=\"" + stylesheet + "\"></HEAD><BODY>");

	int max    = da.maxIdLength();
	int maxlen = da.maxLength();

	if (linelength == -1) {
	    linelength = da.maxLength();
	}
	int start = 0;

	while (start < maxlen) {
	    printChunk(start,start+linelength-1);
	    System.out.println("<br>\n");
	    start += linelength;
	}
    }

    public void printChunk(int start, int end) {
	int max = da.maxIdLength();

	for (int i = 0; i < da.size(); i++) {
	    DrawableSequence ds = da.ds[i];
	    String  prev        = null;
	    String id           = ds.name;

	    if (ds.type > 0) {
                if (ds.type == Sequence.STRUCTURE) {
		  id = id + "-SS";
                } else if (ds.type == Sequence.ACTIVESITE) {
                  id = id + "-AS";
                } else if (ds.type == Sequence.SURFACC) {
                  id = id + "-SA";
                }
	    }

	    int    pad          = max - id.length() + 1;
	    String padstr       = "";

	    for (int j = 0; j < pad; j++) {
		padstr = padstr + "&nbsp;";
	    }

	    System.out.print("<b>" + id + padstr + "&nbsp;&nbsp;</b>");

	    int tmpend = end;

	    if (ds.getSequence().length() < end) {
		tmpend = ds.getSequence().length();
	    }

	    for (int j = start; j < tmpend; j++) {
		Color  c   = (Color)ds.boxColour.elementAt(j);
		String s   = ds.getSequence().substring(j,j+1);
		String col = (String)styles.get(c);

		if (col != null && col != prev) {
		    if (j != start) {
			System.out.print("</b><b ID=\"" + col + "\">" + s);
		    } else {
			System.out.print("<b ID=\"" + col + "\">" + s);
		    }
		} else if (col == null && col != prev) {
		    if (j == start) {
			System.out.print("<b>" + s);
		    } else {
			System.out.print("</b><b>" + s);
		    }
		    
		} else  {
		    if (j == start) {
			System.out.print("<b>" + s);
		    } else {
			System.out.print(s);
		    }
		}
		prev = col;
	    }
	    System.out.println("</b><br>");

	    if (ds.score != null && ds.score.length > 0) {
		Vector scores = ds.score[0];
		if (scores != null) {
		    System.out.print("<b>" + ds.name + padstr + "&nbsp;&nbsp;</b>");
		    
		    for (int k = start; k < tmpend; k++) {
			int score = (int)((Double)scores.elementAt(k)).doubleValue();
			System.out.print("<b>" + score + "</b>");
		    }
		    System.out.println("<br>");
		}
	    }
	}
    }

    public static Hashtable parseArgs(String[] args) {
	Hashtable hash = new Hashtable();

	int i=0;

	while (i < args.length) {
	    if (args[i].indexOf("-") == 0) {
		if (i < (args.length-1) && args[i+1].indexOf("-") != 0) {
		    hash.put(args[i],args[i+1]);
		    i+=2;
		} else {
		    hash.put(args[i],"1");
		}
	    } else {
		System.err.println("Unrecognised option " + args[i]);
		i++;
	    }
	}
	return hash;
    }
	    

    public static void usage() {
	System.out.println("Usage: java jalview.Jhtml -infile  <myfile.aln> [-format <format>] [-linelength <linelength>] [-stylesheet <stylesheet>]\n");
	System.out.println("    Format = MSF CLUSTAL PIR FASTA BLC");
	System.exit(0);
    }

    public static void main(String[] args) {
	Hashtable arghash = parseArgs(args);
	int linelength = 72;

	String format     = "MSF";
	String stylesheet = "Jtml.css";
	String infile     = "";

	if (arghash.get("-help") != null) {
	    usage();
	}

	if (arghash.get("-infile") != null) {
	    infile = (String)arghash.get("-infile");
	} else {
	    usage();
	}
	if (arghash.get("-format") != null) {
	    format = (String)arghash.get("-format");
	}

	if (arghash.get("-stylesheet") != null) {
	    stylesheet = (String)arghash.get("-stylesheet");
	}

	if (arghash.get("-linelength") != null) {
	    linelength = Integer.parseInt((String)arghash.get("-linelength"));
	}

	DrawableSequence[] s = FormatAdapter.toDrawableSequence(FormatAdapter.read(infile,"File",format));      

	Vector markup = new Vector();
	Vector ids    = new Vector();
	Vector seqs   = new Vector();

	DrawableAlignment da = new DrawableAlignment(s);

	// Sort the markups from the real seuqences
	// Assumes the real sequence comes first always

	for (int i = 0; i < s.length; i++) {
            String name = s[i].getName();
            int pos = name.length() - 3;

            if (s[i].type != 0) {
                markup.addElement(s[i]);
	    } else {
		seqs.addElement(s[i]);
		ids.addElement(s[i].getName());
	    }
	}

	//Now make a sequence group out of the markup
	SequenceGroup sg = new SequenceGroup();
	SequenceGroup seqgroup = da.findGroup((Sequence)seqs.elementAt(0));

	da.groups.addElement(sg);

	for (int i = 0; i < markup.size(); i++) {
	    da.removeFromGroup(seqgroup,(Sequence)markup.elementAt(i));
	    da.addToGroup(sg,(Sequence)markup.elementAt(i));
	}
	
	da.percentIdentity(seqs);

	ClustalxColourScheme cs = new ClustalxColourScheme(da.cons2,seqgroup.sequences.size());
        
	sg.colourScheme = new SecondaryColourScheme();
	seqgroup.colourScheme = cs;

	da.setColourScheme(seqgroup);
	da.setColourScheme(sg);

	Jtml jtml = new Jtml(da);
	
	if (linelength == -1) {
	    linelength = da.maxLength();
	}
	jtml.getHtml(linelength,stylesheet);
     
	System.exit(0);
  }
}
  






