/* Jalview - a java multiple alignment editor
 * Copyright (C) 1998  Michele Clamp
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package jalview;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class MailPopup extends Popup {
  TextField tf;
  Label tfLabel;
  boolean postscript;
  Mail mail;
  Label format;
  Choice f;
  String mailServer;

  public MailPopup(Frame parent,String title,String type,String mailServer) {
    super(parent,title);

    this.mailServer = mailServer;

    tf = new TextField(40);
    tfLabel = new Label("Mail address : ");

    format = new Label("Alignment format");
    f = new Choice();
    f.addItem("MSF");
    f.addItem("Fasta");
    f.addItem("Clustal");
    f.addItem("PIR");
    f.addItem("BLC");

    gbc.fill = GridBagConstraints.NONE; 

    gbc.insets = new Insets(20,20,20,20);

    add(tfLabel,gb,gbc,0,0,1,1);
    add(tf,gb,gbc,1,0,4,1);
    add(apply,gb,gbc,0,2,1,1);
    add(close,gb,gbc,1,2,1,1);


    if (type.equals("PS")) {
      this.postscript = true;
    } else {
      add(format,gb,gbc,0,1,1,1);
      add(f,gb,gbc,1,1,1,1);
    
      this.postscript = false;
    }
    this.pack();
    this.show();
    
  }

  public MailPopup(Frame parent,String title,String mailServer) {
    super(parent,title);

    this.mailServer = mailServer;
    tf = new TextField(40);
    tfLabel = new Label("Mail address : ");
    format = new Label("Alignment format");
    f = new Choice();
    f.addItem("MSF");
    f.addItem("Fasta");
    f.addItem("Clustal");
    f.addItem("PIR");
    f.addItem("BLC");

    gbc.fill = GridBagConstraints.NONE; 

    gbc.insets = new Insets(20,20,20,20);

    add(tfLabel,gb,gbc,0,0,1,1);
    add(tf,gb,gbc,1,0,4,1);
    add(format,gb,gbc,0,1,1,1);
    add(f,gb,gbc,1,1,1,1);
    add(apply,gb,gbc,0,2,1,1);
    add(close,gb,gbc,1,2,1,1);

    this.pack();
    this.show();
  }
  
  public boolean handleEvent(Event evt) {
    if (evt.target == apply && evt.id == 1001) {
      String address = tf.getText();

      // Do some checks on the mail address
      if (address.indexOf('@') == -1) {
	System.out.println("Invalid mail address (enter name@my.email.server )");
	this.hide();
	this.dispose();
      } else {
	System.out.println("Mail address " + address) ;

	//Pfam things
	if (parent instanceof AlignFrame) {

	  Sequence[] s = ((AlignFrame)parent).ap.seqPanel.align.sequences;

	  mail = new Mail();

	  // Send to the recipient running the applet
	  String recipient = address;
	  String author = "<pfam@pfam.com>";
	  String subject = "Pfam alignment";
	  String text = "";
	  System.out.println("Mail Server = " + mailServer);
	  mail.send(mailServer,recipient,author,subject,text);

	  sendText(s);

	  mail.finish();

	  this.hide();
	  this.dispose();
	}
	
      }
      return true;
    } else {
      return super.handleEvent(evt);
    }
  }

  public void sendText(Sequence[] s) {
    String text = "";

    if (postscript == true) {
      if (parent instanceof AlignFrame) {
	Postscript p = new Postscript((AlignFrame)parent,PostscriptProperties.PORTRAIT,mail.out);
      } else {
	System.out.println("parent not instanceof AlignFrame");
      }
    } else {
      if (f.getSelectedItem().equals("MSF")) {
	try {
	  if (parent instanceof AlignFrame) {
	    AlignFrame af = (AlignFrame)parent;
	    text = MSFfile.print(af.ap.seqPanel.align.sequences);
	  } else {
	    System.out.println("Error : parent isn't Alignment Frame");
	  }
	} catch (Exception ex) {
	  System.out.println("Exception in MailPopup: " + ex);
	}
      } else if (f.getSelectedItem().equals("Fasta")) {
	try {
	  if (parent instanceof AlignFrame) {
	    AlignFrame af = (AlignFrame)parent;
	    text = FastaFile.print(af.ap.seqPanel.align.sequences,72);
	  } else {
	    System.out.println("Error : parent isn't Alignment Frame");
	  }
	} catch (Exception ex) {
	  System.out.println("Exception in MailPopup: " + ex);
	}
      } else if (f.getSelectedItem().equals("Clustal")) {
	try {
	  if (parent instanceof AlignFrame) {
	    AlignFrame af = (AlignFrame)parent;
	    text = ClustalFile.print(af.ap.seqPanel.align.sequences);
	  } else {
	    System.out.println("Error : parent isn't Alignment Frame");
	  }
	} catch (Exception ex) {
	  System.out.println("Exception in MailPopup: " + ex);
	}
      } else if (f.getSelectedItem().equals("BLC")) {
	try {
	  if (parent instanceof AlignFrame) {
	    AlignFrame af = (AlignFrame)parent;
	    text = BLCFile.print(af.ap.seqPanel.align.sequences);
	  } else {
	    System.out.println("Error : parent isn't Alignment Frame");
	  }
	} catch (Exception ex) {
	  System.out.println("Exception in MailPopup: " + ex);
	}
      } else if (f.getSelectedItem().equals("PIR")) {
	try {
	  if (parent instanceof AlignFrame) {
	    AlignFrame af = (AlignFrame)parent;
	    text = PIRFile.print(af.ap.seqPanel.align.sequences,72);
	  } else {
	    System.out.println("Error : parent isn't Alignment Frame");
	  }
	  
	} catch (Exception ex) {
	  System.out.println("Exception in MailPopup: " + ex);
	}
      } else {
	System.out.println("Not yet supported");
      }

      try {
	mail.send(mail.out,text);
      }catch (Exception e) {
      }
    }
  }

  public static void main(String[] args) {
    Frame f = new Frame("frame");
 
    f.resize(100,100);
    f.show();


  }
}
