package jalview;
import java.awt.*;
import java.io.*;
import java.util.*;

public class PCA implements Runnable {
  Matrix m;
  Matrix symm;
  Matrix m2;

  double[] eigenvalue;
  Matrix eigenvector;
  ProgressBar pb;
  Sequence[] s;
  Vector resscores;

  public PCA(Matrix m) {
    this.m = m;
  }

  public PCA(Sequence[] s,ProgressBar pb) {
    this.pb = pb;
    this.s = s;
    Runtime rt = Runtime.getRuntime();
    
    BinarySequence[] bs = new BinarySequence[s.length];
    int ii = 0;

    while (ii < s.length && s[ii] != null) {
  
      bs[ii] = new BinarySequence(s[ii]);
      bs[ii].encode();
      ii++;
    }
   
    BinarySequence[] bs2 = new BinarySequence[s.length];
    ii = 0;
    while (ii < s.length && s[ii] != null) {
  
      bs2[ii] = new BinarySequence(s[ii]);
      bs2[ii].blosumEncode();
      ii++;
    }
   
    
    int count=0;
      while (count < bs.length && bs[count] != null) {
     count++;
    }

    double[][] seqmat = new double[count][bs[0].dbinary.length];
    double[][] seqmat2 = new double[count][bs2[0].dbinary.length];
    int i=0;

    while (i < count) {
      seqmat[i] = bs[i].dbinary;
      seqmat2[i] = bs2[i].dbinary;
      i++;
    }


    m  = new Matrix(seqmat,count,bs[0].dbinary.length);
    m2 = new Matrix(seqmat2,count,bs2[0].dbinary.length);
    
    printMemory(rt);
  }
  
  public static void printMemory(Runtime rt) {
    System.out.println("Free memory = " + rt.freeMemory());
  }
  
  public double[] getEigenvector(int i) {
    return eigenvector.getColumn(i);
  }

  public double getEigenvalue(int i) {
    return eigenvector.d[i];
  }
  public float[][] getComponents(int l, int n, int mm) {
    return getComponents(l,n,mm,1);
  }
  public float[][] getComponents(int l, int n, int mm, float factor) {
    float[][] out = new float[m.rows][3];

    for (int i = 0; i < m.rows;i++) {
      out[i][0] = (float)component(i,l)*factor; 
      out[i][1] = (float)component(i,n)*factor; 
      out[i][2] = (float)component(i,mm)*factor; 
    }
    return out;
  }

  public double[] component(int n) {
    // n = index of eigenvector
    double[] out = new double[m.rows];

    for (int i=0; i < m.rows; i++) {
      out[i] = component(i,n);
    }
    return out;
  }
  public double component(int row, int n) {
    double out = 0.0;
    
    for (int i = 0; i < symm.cols; i++) {
      out += symm.value[row][i] * eigenvector.value[i][n];
    }
    return out/eigenvector.d[n];
  }

  public void checkEigenvector(int n,PrintStream ps) {
    ps.println(" --- Eigenvector " + n  + " --- ");
    
    double[] eigenv = eigenvector.getColumn(n);

    for (int i=0; i < eigenv.length;i++) {
      Format.print(ps,"%15.4f",eigenv[i]);
    }

    System.out.println();      
    
    double[] neigenv = symm.vectorPostMultiply(eigenv);
    ps.println(" --- symmat * eigenv / lambda --- ");
    if (eigenvector.d[n] > 1e-4) {
      for (int i=0; i < neigenv.length;i++) {
	Format.print(ps,"%15.4f",neigenv[i]/eigenvector.d[n]);
      }
    }
      System.out.println();      
  }
  public void updateBar(int value) {
      if (pb != null) {
	  System.out.println("UPdating bar " + value);
	  pb.updateBar(value);
      }
  }

  public void run() {
    Matrix mt = m.transpose();

    updateBar(5);

    eigenvector = mt.preMultiply(m2);

    updateBar(10);

    symm = eigenvector.copy();
    
    updateBar(20);

    TextArea ta = new TextArea(25,72);
    TextAreaPrintStream taps = new TextAreaPrintStream(System.out,ta);

    Frame f = new Frame("PCA output");
    f.resize(500,500);
    f.setLayout(new BorderLayout());
    f.add("Center",ta);
    f.show();
    //    symm.print(taps);

    long tstart = System.currentTimeMillis();

    eigenvector.tred(pb);

    long tend = System.currentTimeMillis();

    taps.println("Time take for tred = " + (tend-tstart) + "ms");
    taps.println(" ---Tridiag transform matrix ---");
    
    taps.println(" --- D vector ---");
    eigenvector.printD(taps);
    taps.println();   
    taps.println(" --- E vector ---");
    eigenvector.printE(taps);
    taps.println();   
    

    // Now produce the diagonalization matrix
    tstart = System.currentTimeMillis();
    eigenvector.tqli(pb);
    tend = System.currentTimeMillis();

    if (pb != null) {pb.updateBar(100);}

    taps.println("Time take for tqli = " + (tend-tstart) + " ms");
    
    //System.out.println(" --- New diagonalization matrix ---");
    
    taps.println(" --- Eigenvalues ---");
    eigenvector.printD(taps);

    //System.out.println();  
 
    //          for (int i=0; i < eigenvector.cols; i++) {
    //          checkEigenvector(i,taps);
    //          taps.println();
 

    //}
    
    //    taps.println();
    //    taps.println("Transformed sequences = ");

    //   Matrix trans =  m.preMultiply(eigenvector);
    //trans.print(taps);


    // I want to do something with these eigenvectors
    // Take the first eigenvector and the alignment in colums
    // Alignment is in m

    resscores = new Vector();

    for (int k = 0; k < eigenvector.cols; k++) {
	
	double bigeig[] = eigenvector.getColumn(k);
	double scores[][] = new double[s.length][s[0].getSequence().length()];
	
	double max = -100000;
	double min =  100000;
	
	Hashtable[] alnhash = new Hashtable[s[0].getSequence().length()];
	
	// Loop over all columns in the alignment
	for (int i=0; i < s[0].getSequence().length(); i++) {
	    
	    // taps.println("Column number " + i);
	    alnhash[i]       = new Hashtable();
	    
	    // Loop over all residues in the column
	    double newval = 0;
	    for (int j = 0;j < s.length; j++) {
		String residue;
		//  System.out.println(i + " " + s[j].getSequence().length());
		if (s[j].getSequence().length() > i) {
		    residue = (String)(s[j].getSequence().substring(i,i+1));
		} else {
		    residue = (String)(s[j].getSequence().substring(i));
		}
		
		double val =0;
		if (alnhash[i].containsKey(residue)) {
		    val = ((Double)alnhash[i].get(residue)).doubleValue();
		}
		
		val = val + bigeig[j];
		//	    taps.println("VAL " + val);
		alnhash[i].put(residue,new Double(val));
		
	    }
	    
	    // We should now have a vector with values for each residue
	    Enumeration e = alnhash[i].keys();
	    
	    while (e.hasMoreElements()) {
		String residue = (String)e.nextElement();
		double val = ((Double)alnhash[i].get(residue)).doubleValue();

		if (val < min) {min = val;}
		if (val > max) {max = val;}

		alnhash[i].put(residue,new Double(val));
	    }
	}

	taps.println("MAXMIN " + max + " " + min);
	taps.println("");
	// Now loop over all residues and make scores
	for (int j = 0;j < s.length; j++) {
	    for (int i=0; i < s[j].getSequence().length(); i++) {
		
		Hashtable tmp = alnhash[i];
		String res;
		
		if (i < s[j].getSequence().length()) {
		    res = s[j].getSequence().substring(i,i+1);
		} else {
		    res = s[j].getSequence().substring(i);
		}
		
		float score = (float)((Double)tmp.get(res)).floatValue();
		
		score = (score - (float)min)* 10/(float)(max - min);
		scores[j][i] = score;
		//	System.out.println(j + " " + i + " " + scores[j][i]);
		//		s[j].setScore(i,score,0);
	    }
	}
	resscores.addElement(scores);
    }

  }
}


