/* Jalview - a java multiple alignment editor
 * Copyright (C) 1998  Michele Clamp
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package jalview;
import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.Font;

public class PfamFile extends FileParse {
  int noSeqs;
  int maxLength = 0;

  Hashtable myHash;  //hashtable containing the sequences
  Vector seqs;  //Vector of Sequences
  Vector headers;
  Vector ids;

  Vector words = new Vector();  //Stores the words in a line after splitting

  long start;
  long end;

  public PfamFile(String inStr) {
    myHash  = new Hashtable();
    seqs    = new Vector();
    headers = new Vector();
    ids     = new Vector();

    readLines(inStr);
    System.out.println(noLines);
    System.out.println(lineArray.size());
    parse();
  }

  public PfamFile(String inFile, String type) throws IOException {
    //Read in the file first
    super(inFile,type);
    
    myHash = new Hashtable();
    seqs = new Vector();
    headers = new Vector();
    ids = new Vector();
    //Read lines from file
    System.err.print("Reading file....");
    start = System.currentTimeMillis();
    readLines();
    end = System.currentTimeMillis();
    System.err.println("done");
    System.err.println("Total time taken = " + (end-start) + "ms");

    System.err.println("Parsing file....");
    start = System.currentTimeMillis();
    parse();
  }
  public void parse() {
    //Parse lines
    int i = 0;  //line counter
    
    Vector ids = new Vector();
    Hashtable fseqtypes = new Hashtable();
    Hashtable ftypes    = new Hashtable();
    Hashtable featureStrs = new Hashtable();

    //Loop over lines in file
    for (i = 0; i < noLines; i++) {
	if (lineArray.elementAt(i).toString().indexOf(" ") != 0) {
	  if (lineArray.elementAt(i).toString().indexOf("#") != 0) {
	      StringTokenizer str = new StringTokenizer(lineArray.elementAt(i).toString()," ");
	      String id = "";
	      
	      if (str.hasMoreTokens()) {
		  id = str.nextToken();
		  
		  StringBuffer tempseq = new StringBuffer();
		  if (myHash.containsKey(id)) {
		      tempseq = new StringBuffer(myHash.get(id).toString());
		  }
		  if (!(headers.contains(id))) {
		      headers.addElement(id);
		  }
		  
		  //loop through the rest of the words
		  //  while (str.hasMoreTokens()) {	  
		  tempseq.append(str.nextToken());
		  // }
		  //put the sequence back in the hash
		  myHash.put(id,tempseq.toString());
	      }
	  } else if (lineArray.elementAt(i).toString().indexOf("#=") == 0){
	      // We have a feature line
	      StringTokenizer str = new StringTokenizer(lineArray.elementAt(i).toString()," ");
	      String fseqtype;
	      String id;
	      String ftype;
	      String featureStr = "";

	      if (str.hasMoreTokens()) {
		  fseqtype = str.nextToken();
		  
		  //  System.out.println("Feature type is " + fseqtype);
		  
		  id    = str.nextToken();
		  ftype = str.nextToken();
		  
		  if (!ids.contains(id)) {
		      ids.addElement(id);
		  }

		  if (fseqtype.equals("#=GR")) {
		      ftypes.put(id + ftype,ftype);
		      fseqtypes.put(id + ftype,fseqtype);

		      //   System.out.println("Found residue feature");
		      featureStr = str.nextToken();

		      featureStrs.put(id + ftype,featureStr);

		  } else if (fseqtype.equals("#=GS")) {
		      ftypes.put(id + ftype,ftype);
		      fseqtypes.put(id + ftype,fseqtype);

		      //System.out.println("Found sequence feature");
		      while (str.hasMoreTokens()) {
			  featureStr += str.nextToken();
		      }

		      featureStrs.put(id + ftype,featureStr);

		  } else if (fseqtype.equals("#=GC")) {
		      ftypes.put(id + "GC",ftype);
		      fseqtypes.put(id + "GC",fseqtype);

		      //System.out.println("Consensus feature");
		      //		      id = str.nextToken();
		      //System.out.println("id is " + id);

		      //featureStr = str.nextToken();
		      //System.out.println("str is " + featureStr);
		      //featureStrs.put(id + "GC",featureStr);

		  } else {
		      System.err.println("Unrecognised feature type " + fseqtype);
		  }
	      }
	      
	  }
      }
    }
    
    this.noSeqs = headers.size();
      
    //Add sequences to the hash
    for (i = 0; i < headers.size(); i++ ){
	
      if ( myHash.get(headers.elementAt(i)) != null) {
	if (maxLength <  myHash.get(headers.elementAt(i)).toString().length() ) {
	  maxLength =  myHash.get(headers.elementAt(i)).toString().length();
	}
	String head =  headers.elementAt(i).toString();
	int start = 1;
	int end =  myHash.get(headers.elementAt(i)).toString().length();
	

	if (head.indexOf("/") > 0 ) { 
	  StringTokenizer st = new StringTokenizer(head,"/");
	  if (st.countTokens() == 2) {
	    ids.addElement(st.nextToken());
	    String tmp = st.nextToken();
	    st = new StringTokenizer(tmp,"-");
	    if (st.countTokens() == 2) {
	      start = Integer.valueOf(st.nextToken()).intValue();
	      end = Integer.valueOf(st.nextToken()).intValue();
	    } else {
	      start = -1;
	      end = -1;
	    }
	  } else {
	    ids.addElement(headers.elementAt(i));
	  }
	} else {
	  ids.addElement(headers.elementAt(i));
	}
	
	//	System.out.println(headers.elementAt(i));
	//System.out.println(ids.elementAt(i));
	//System.out.println( myHash.get(headers.elementAt(i).toString()).toString());
	if (start != -1 && end != -1) {
	    Sequence newSeq = new Sequence(ids.elementAt(i).toString(),
					   myHash.get(headers.elementAt(i).toString()).toString(),start,end);
	    seqs.addElement(newSeq);

	    // Now add in the features for the appropriate sequence
	    // Add in a sequence for the secondary structure
	    // Add in the scores to that sequence for the secondary structure scores
	    
	    String idStr = newSeq.getName();

	    if (ids.contains(idStr)) {

		if (ftypes.containsKey(idStr + "SS")) {
		    String ssstr = (String)featureStrs.get(idStr + "SS");
		    Sequence ssSeq = new Sequence(idStr,ssstr,newSeq.getStart(),newSeq.getEnd());
		    ssSeq.type = Sequence.STRUCTURE;
		    seqs.addElement(ssSeq);
                }
		    // Now add the scores
		if (featureStrs.containsKey(idStr + "SA")) {
		  String scoreStr = (String)featureStrs.get(idStr + "SA");
		  Sequence scoreSeq = new Sequence(idStr,scoreStr,newSeq.getStart(),newSeq.getEnd());
		  scoreSeq.type = Sequence.SURFACC;
		  seqs.addElement(scoreSeq);
		}
		if (featureStrs.containsKey(idStr + "AS")) {
		  String asStr = (String)featureStrs.get(idStr + "AS");
		  Sequence asSeq = new Sequence(idStr,asStr,newSeq.getStart(),newSeq.getEnd());
		  asSeq.type = Sequence.ACTIVESITE;
		  seqs.addElement(asSeq);
		}
		
	    }
	} else {
	    Sequence newSeq = new Sequence(ids.elementAt(i).toString(),
					   myHash.get(headers.elementAt(i).toString()).toString(),1,
					   myHash.get(headers.elementAt(i).toString()).toString().length());

	    
	    seqs.addElement(newSeq);
	    
	    // Now add in the features for the appropriate sequence
	    // Add in a sequence for the secondary structure
	    // Add in the scores to that sequence for the secondary structure scores
	    
	    String idStr = newSeq.getName() + "/" + newSeq.getStart() +"-" + newSeq.getEnd();
	    
	    if (ids.contains(idStr)) {
		// first look for secondary structure
		if (ftypes.contains(idStr + "SS")) {
		    String ssstr = (String)featureStrs.get(idStr + "SS");
		    Sequence ssSeq = new Sequence(idStr,ssstr,newSeq.getStart(),newSeq.getEnd());
		    ssSeq.type = Sequence.STRUCTURE;
		    seqs.addElement(ssSeq);
		    System.err.println("Tyhpe " + ssSeq.type);
                 }
		// Now add the scores
		if (featureStrs.containsKey(idStr + "SA")) {
		  String scoreStr = (String)featureStrs.get(idStr + "SA");
		  Sequence scoreSeq = new Sequence(idStr,scoreStr,newSeq.getStart(),newSeq.getEnd());
		  scoreSeq.type = Sequence.SURFACC;
		  seqs.addElement(scoreSeq);
		}
		if (featureStrs.containsKey(idStr + "AS")) {
		  String asStr = (String)featureStrs.get(idStr + "AS");
		  Sequence asSeq = new Sequence(idStr,asStr,newSeq.getStart(),newSeq.getEnd());
		  asSeq.type = Sequence.ACTIVESITE;
		  seqs.addElement(asSeq);
		}
	    }
	}
      } else {
	  System.err.println("Can't find sequence for " + headers.elementAt(i));
      }
    }
    
    end = System.currentTimeMillis();
    System.err.println("done");
    System.err.println("Total time taken = " + (end-start) + "ms");
  }
 
  public static String print(Sequence[] s) {
    StringBuffer out = new StringBuffer("");

    int max = 0;
    int maxid = 0;
    
    int i = 0;

    while (i < s.length && s[i] != null) {
      String tmp = s[i].getName() + "/" + s[i].start + "-" + s[i].end;

      if (s[i].getSequence().length() > max) {
	max = s[i].getSequence().length();
      }
      if (tmp.length() > maxid) {
	maxid = tmp.length();
      }
      i++;
    }

    if (maxid < 15) { maxid = 15; }
    
    int j = 0;
    while ( j < s.length && s[j] != null) {
      out.append( new Format("%-" + maxid + "s").form(s[j].getName() + "/" + s[j].start + "-" + s[j].end ) + " ");
	
      out.append(s[j].getSequence() + "\n");
      j++;
    }
    out.append("\n");
    
    return out.toString();
  }


  public static void main(String[] args) {
      String file = args[0];
      try {
	  PfamFile fp = new PfamFile(file,"File");
      } catch (Exception e) {
	  System.out.println(e);
      }
  }

} 







